@@ -9,7 +9,8 @@ class install_depends_py():
         self.results = {}
         self.stdout = {}
         self.stderr = {}
-        self.install_results = {}   
+        self.install_results = {}
+        self.test_hardware = self.test_hardware   
         if "test_ipfs_accelerate" not in globals() and "test_ipfs_accelerate" not in list(self.resources.keys()):
             import test_ipfs_accelerate
             self.test_ipfs_accelerate = test_ipfs_accelerate.test_ipfs_accelerate(resources, metadata)
@@ -18,6 +19,45 @@ class install_depends_py():
         elif "test_ipfs_accelerate" in globals():
             self.test_ipfs_accelerate = test_ipfs_accelerate(resources, metadata)
 
+    async def test_hardware(self):
+        install_file_hash = None
+        test_results_file = None
+        install_depends_filename = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "install_depends", "install_depends.py")
+        if os.path.exists(install_depends_filename):
+            ## get the sha256 hash of the file
+            sha256 = hashlib.sha256()
+            with open(install_depends_filename, "rb") as f:
+                for byte_block in iter(lambda: f.read(4096),b""):
+                    sha256.update(byte_block)
+            install_file_hash = sha256.hexdigest()
+            test_results_file = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))),"test", install_file_hash + ".json")
+            if os.path.exists(test_results_file):
+                try:
+                    with open(test_results_file, "r") as f:
+                        test_results = json.load(f)
+                        return test_results
+                except Exception as e:
+                    try:
+                        test_results = await self.install_depends.test_hardware()
+                        with open(test_results_file, "w") as f:
+                            json.dump(test_results, f)
+                        return test_results
+                    except Exception as e:
+                        print(e)
+                        return e
+            else:
+                try:
+                    test_results = await self.install_depends.test_hardware()
+                    with open(test_results_file, "w") as f:
+                        json.dump(test_results, f)
+                    return test_results
+                except Exception as e:
+                    print(e)
+                    return e
+        else: 
+            raise ValueError("install_depends.py not found")
+        return test_results
+    
     
     async def install(self, resources=None):        
         if resources is None:
@@ -319,7 +359,7 @@ class install_depends_py():
    
     async def test_huggingface_optimum_amx(self):
         import optimum
-        test_optimum_amx_cmd = optimum.AMX()
+        test_optimum_amx_cmd = "python3 -c 'import transformers; print(optimum.__version__)'"
         try:
             test_optimum_amx = subprocess.check_output(test_optimum_amx_cmd, shell=True).decode("utf-8")
             if type(test_optimum_amx) == str and type(test_optimum_amx) != ValueError:
@@ -738,16 +778,16 @@ class install_depends_py():
             print(f"Failed to install Optimum IPEX: {e.stderr}")
         return install_results
     
-    async def install_huggingface_optimum_amx(self):
-        install_results = {}
-        try:
-            install_cmd = "pip install --upgrade --upgrade-strategy eager optimum[amx]"
-            result = subprocess.run(install_cmd, check=True, capture_output=True, text=True)
-            install_results["install_optimum_amx"] = result.stdout
-        except subprocess.CalledProcessError as e:
-            install_results["install_optimum_amx"] = e.stderr
-            print(f"Failed to install Optimum AMX: {e.stderr}")
-        return install_results            
+    # async def install_huggingface_optimum_amx(self):
+    #     install_results = {}
+    #     try:
+    #         install_cmd = "pip install --upgrade --upgrade-strategy eager optimum[amx]"
+    #         result = subprocess.run(install_cmd, check=True, capture_output=True, text=True)
+    #         install_results["install_optimum_amx"] = result.stdout
+    #     except subprocess.CalledProcessError as e:
+    #         install_results["install_optimum_amx"] = e.stderr
+    #         print(f"Failed to install Optimum AMX: {e.stderr}")
+    #     return install_results            
 
     async def install_huggingface_optimum_habana(self):
         install_results = {}
@@ -1022,17 +1062,17 @@ class install_depends_py():
         optimum_amx_install = None
         optimum_amx_test = None
         
-        try:
-            optimum_amx_test = await self.test_huggingface_optimum_amx()
-        except Exception as e:
-            optimum_amx_test = e
-            print(e)
-            try:
-                optimum_amx_install = await self.install_huggingface_optimum_amx()
-            except Exception as e:
-                optimum_amx_install = e
-                print(e)
-            pass
+        # try:
+        #     optimum_amx_test = await self.test_huggingface_optimum_amx()
+        # except Exception as e:
+        #     optimum_amx_test = e
+        #     print(e)
+        #     try:
+        #         # optimum_amx_install = await self.install_huggingface_optimum_amx()
+        #     except Exception as e:
+        #         optimum_amx_install = e
+        #         print(e)
+        #     pass
         
         try:
             onnx_test = await self.test_onnx()
